package cn.com.ctop.shiwan.modules;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * TODO
 *
 * @ClassName ZipUtil
 * @Author ZHAOXA
 * @date 2020-10-23 10:05
 */
@Slf4j
public class FileUtil {

    /**
     * 转MultipartFile格式文件
     *
     * @param
     * @return org.springframework.web.multipart.MultipartFile
     * @throws
     * @author ZHAOXA
     */
    public static MultipartFile getMulFileByPath(String picPath) {
        FileItem fileItem = createFileItem(picPath);
        MultipartFile mfile = new CommonsMultipartFile(fileItem);
        return mfile;
    }

    private static FileItem createFileItem(String filePath) {
        FileItemFactory factory = new DiskFileItemFactory(16, null);
        String textFieldName = "textField";
        int num = filePath.lastIndexOf("/");
        String fileName = filePath.substring(num);
        FileItem item = factory.createItem(textFieldName, "text/plain", true,
                fileName);
        File newfile = new File(filePath);
        int bytesRead = 0;
        byte[] buffer = new byte[8192];
        try {
            FileInputStream fis = new FileInputStream(newfile);
            OutputStream os = item.getOutputStream();
            while ((bytesRead = fis.read(buffer, 0, 8192))
                    != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            fis.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return item;
    }


    /**
     * 下载文件到本地
     *
     * @param locationPath 本地路径
     * @param fileUrl      文件下载路径
     * @param fileName     下载后的名称含后缀
     * @return java.lang.String
     * @throws
     * @author ZHAOXA
     */
    public static String writeFiles(String locationPath, String fileUrl, String fileName) {
        InputStream is = null;
        OutputStream os = null;
        Date date = new Date();
        String realPath = null;
        try {
            URL url = new URL(fileUrl);
            is = url.openStream();
            File file = new File(locationPath);
            if (!file.exists()) {
                boolean mkdirFlag = file.mkdirs();
                if (!mkdirFlag) {
                    log.error("[ ========== 下载，创建临时文件夹失败 ========== ]");
                    return null;
                }
            }
            realPath = locationPath.concat(fileName);
            os = new FileOutputStream(realPath);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = is.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
                os.flush();
            }
            os.close();
            is.close();
        } catch (Exception e) {
            log.error(" ========== 文件下载到本地异常：", e);
            return null;
        }
        return realPath;
    }


    /**
     * //创建zip方法，其中的参数ZipFileName是压缩后文件，inputFile要压缩的文件。
     *
     * @param
     * @return void
     * @throws
     * @author ZHAOXA
     */
    public static void zip(String ZipFileName, File inputFile) throws IOException {
        ZipOutputStream out = new ZipOutputStream(new FileOutputStream(ZipFileName));//创建了输出流，这里ZipOutputStream()的参数是FileOutPutStream的原因是因为ZIP压缩的对象是文件。
        zip(out, inputFile, "");//调用zip方法，实现了zip的重载。这次的参数是本方法中定义过的输出流out，要压缩的文件：inputFile以及文件最后的存储目录base
        log.info("压缩中....");
        out.close();
        log.info("压缩完成！");
    }

    public static void zip(ZipOutputStream out, File f, String base) throws IOException {//方法重载
        if (f.isDirectory()) {//判断要压缩的文件是否是文件夹
            File[] fl = f.listFiles();//如果是文件夹就对把该文件夹中的文件的路径以文件数组的形式获得
            if (base.length() != 0) {//判断最后要存储的路径是否已存在
                out.putNextEntry(new ZipEntry(base + "/"));//如果已经存在，写入此目录的entry
            }
            for (int i = 0; i < fl.length; i++) {//遍历文件数组
                zip(out, fl[i], base + fl[i]);//将文件数组中的文件添加到已经创建的ZIP中。
            }
        } else {
            out.putNextEntry(new ZipEntry(base));//如果base为空。那么就创建新的进入点
            //创建FileInputStream对象
            FileInputStream in = new FileInputStream(f);//创建该输入流的原因是为了后边判断该流是否到达尾部
            int b;
            System.out.println(base);
            while ((b = in.read()) != -1) {
                out.write(b);//在没有到达尾部的情况下向已经创建的ZIP中添加
            }
            in.close();//关闭流
        }
    }

    /**
     * 文件压缩方法
     *
     * @param list
     * @param zipfilename
     * @param basePath
     * @param vid
     * @throws
     * @Title: listToZip
     * @date 2020年1月20日
     */
    public static String listToZip(List<String> list, String localPath, String zipfilename) {
        long start = System.currentTimeMillis();
        FileInputStream is = null;
        ZipOutputStream zos = null;
        String realPath = localPath + "/newFiles/" + zipfilename;
        try {
            if (list != null && list.size() > 0) {
                File f = new File(localPath, "newFiles");
                if (!f.exists()) {
                    boolean mkdirFlag = f.mkdirs();
                    if (!mkdirFlag) {
                        log.error(" ==========压缩， 创建临时文件夹失败 ========== ");
                        return null;
                    }
                }
                // 创建zip文件输出流
                zos = new ZipOutputStream(new FileOutputStream(new File(realPath)));
                File file = null;
                String path = "";
                for (int i = 0; i < list.size(); i++) {
                    path = list.get(i);
                    // file 拿到待压缩文件
                    file = new File(path);
                    if (file.exists()) {
                        // 创建源文件输入流
                        is = new FileInputStream(file);
                        zos.putNextEntry(new ZipEntry(file.getName()));
                        byte[] buf = new byte[2048];
                        int length = -1;
                        while ((length = is.read(buf)) != -1) {
                            zos.write(buf, 0, length);
                            zos.flush();
                        }
                        zos.closeEntry();
                        is.close();
                    } else {
                        log.warn("[ ======= {} 源文件不存在 ======= ]", path);
                        return null;
                    }
                }
            }
        } catch (Exception e) {
            log.error("[ ==========压缩文件异常 ========== ]", e);
            return null;
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
                if (zos != null) {
                    zos.close();
                }
            } catch (IOException e) {
                log.error("[ ==========关闭IO流失败========== ]", e);
            }
        }
        long end = System.currentTimeMillis();
        log.info("文件压缩完成，耗时：{} ms", (end - start));
        return realPath;
    }


    /**
     * 通过MultipartFile获取本地下载路径
     *
     * @param
     * @return java.lang.String
     * @throws
     * @author ZHAOXA
     */
    public static String approvalFile(MultipartFile filecontent, String path) {
        OutputStream os = null;
        InputStream inputStream = null;
        String fileName = filecontent.getOriginalFilename();
        try {
            inputStream = filecontent.getInputStream();
        } catch (IOException e) {
            log.error("获取文件流异常");
        }
        try {
            byte[] bs = new byte[1024];
            // 读取到的数据长度
            int len;
            // 输出的文件流保存到本地文件
            File tempFile = new File(path);
            if (!tempFile.exists()) {
                tempFile.mkdirs();
            }
            os = new FileOutputStream(tempFile.getPath() + "/" + File.separator + fileName);
            // 开始读取
            while ((len = inputStream.read(bs)) != -1) {
                os.write(bs, 0, len);
            }
        } catch (Exception e) {
            log.error("获取文件的本地路径失败", e);
        } finally {
            // 完毕，关闭所有链接
            try {
                os.close();
                inputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return path + fileName;
    }


    /**
     * @param zipSavePath 压缩好的zip包存放路径
     * @param sourceFile  待压缩的文件（单个文件或者整个文件目录）
     * @return
     * @Description
     * @author xukaixun
     */
    public static String zipCompress(String zipPath, File sourceFile) {
        try {
            //创建zip输出流
            ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(zipPath));
            File[] fileList = sourceFile.listFiles();
            if (fileList.length != 0) {
                for (File file : fileList) {
                    //根据filename判断是否要最外层文件夹
                    compress(zos, file, file.getName());
                }
                zos.close();
            }
        } catch (Exception e) {
            log.error("压缩文件异常: {}, zipSavePath={}, sourceFile={}", e, zipPath, sourceFile.getName());
        }
        return zipPath;
    }

    private static void compress(ZipOutputStream zos, File sourceFile, String fileName) throws Exception {
        if (sourceFile.isDirectory()) {
            //如果是文件夹，取出文件夹中的文件（或子文件夹）
            File[] fileList = sourceFile.listFiles();
            if (fileList.length == 0)//如果文件夹为空，则只需在目的地zip文件中写入一个目录进入点
            {
                zos.putNextEntry(new ZipEntry(fileName + "/"));
            } else//如果文件夹不为空，则递归调用compress，文件夹中的每一个文件（或文件夹）进行压缩
            {
                for (File file : fileList) {
                    compress(zos, file, fileName + "/" + file.getName());
                }
            }
        } else {
            if (!sourceFile.exists()) {
                zos.putNextEntry(new ZipEntry("/"));
                zos.closeEntry();
            } else {
                //单个文件，直接将其压缩到zip包中
                zos.putNextEntry(new ZipEntry(fileName));
                FileInputStream fis = new FileInputStream(sourceFile);
                byte[] buf = new byte[1024];
                int len;
                //将源文件写入到zip文件中
                while ((len = fis.read(buf)) != -1) {
                    zos.write(buf, 0, len);
                }
                zos.closeEntry();
                fis.close();
            }
        }
    }

}
